from bodhi.client.bindings import BodhiClient

import pytest
import mock

from oraculum.utils import health_check
from oraculum import app, CACHE


class TestHealthCheck(object):

    stub_fedora_releases = {
        "oldstable": 96,
        "stable": 97,
        "branched": 98,
        "rawhide": 99,
        "values": [96, 97, 98, 99]
    }

    mocked_basic_health_check_data = {
        "98": {
            "package_a": {
                "noarch": {
                    "reason": ["chromium"],
                    "since": "2020-07-17T07:47:25.219142397Z"
                }
            }
        }
    }

    def health_advanced_releases_stub(self, what):
        if "fedora_releases" in what:
            return self.stub_fedora_releases

        if "health_check_data" in what:
            return self.mocked_basic_health_check_data


    def test_get_data_structure(self, monkeypatch):

        expected_result = {
            "96": {},
            "96-testing": {},
            "97": {},
            "97-testing": {},
            "98": {},
            "98-testing": {},
            "rawhide": {},
        }


        monkeypatch.setattr(CACHE, 'get', mock.MagicMock(return_value=self.stub_fedora_releases))
        monkeypatch.setattr(health_check, 'process_health_check_json', mock.MagicMock(return_value={}))

        assert health_check.get_health_check_data() == expected_result

    def test_get_health_check_user_data(self, monkeypatch):

        expected_result_basic = {
            'package_a': [
                {
                    'problems':{
                        'noarch': {
                            'reason': ['chromium'],
                            'since': '2020-07-17T07:47:25.219142397Z'
                            }
                        },
                    'release': 'Fedora 98',
                    'repo': 'stable'
                }
            ]
        }


        monkeypatch.setattr(CACHE, 'get', self.health_advanced_releases_stub)
        assert health_check.get_health_check_user_data(["package_a"]) == expected_result_basic
        assert health_check.get_health_check_user_data(["package_b"]) == {'package_b': []}
